/*

  Radiator Balance
  by Tim Jackson.1960

  Creadits:
		Arduino.
		<DallasTemperature.h> milesburton.
		<OneWire.h> Paul Stoffregen.


  A simple device to Balance home Radiators, set the differential Temperature.
  Has 2 DS18B20 or equivelent Temperature Sensors placed on the In and Out of a radiator and states if the Lock Vale requires opening or closing.

Found device 0 with address: 28444C2A00008098 These are mine, yours will be different, I just made a note.
Found device 1 with address: 282F573100008003 These are mine, yours will be different, I just made a note.


*/
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
#include <OneWire.h>			//	https://www.pjrc.com/teensy/td_libs_OneWire.html
#include <DallasTemperature.h>	//	https://github.com/milesburton/Arduino-Temperature-Control-Library

/*
	One Wire
		Using Arduino pin 8
*/
#define ONE_WIRE_BUS 8
OneWire OneWireBus(ONE_WIRE_BUS);
/*
	Temperature Sensors.

		TEMPERATURE_DIFFERENTIAL is Temperature (Celsius) that you want the difference in temperature to be between the In and Out pipe of the radiator.
			Recomended is:
				11-12 degrees difference In/Out of Radiator.
				20 degrees difference In/Out Boiler. (dependent on type of boiler)

*/
#define SENSOR_IN					1			//	Swop this value with SENSOR_OUT value if the sensors are marked up wrong for in and out.
#define SENSOR_OUT					0			//	Swop this value with SENSOR_IN value if the sensors are marked up wrong for in and out.
#define TEMPERATURE_DIFFERENTIAL	11.5		//	Change to your prefference.
DallasTemperature Sensors(&OneWireBus);			//	Pass to Dallas Temperature. 
int NumberOfDevices;							//	Should be 2 found on this application.
DeviceAddress Addresses_of_Temperature_Sensors;	//	Stores the addresses of Temperature Sensors found, uint8_t DeviceAddress[8]
float Temperatures_inC[8];						//	I made it 8 becouse DeviceAddress has.
/*
	LCD
		Address = 0x27.
		Charectors per line = 16.
		Number of Lines = 2.
*/
LiquidCrystal_I2C lcd(0x27, 16, 2);
/*
	Configoration and Initiation
*/
void setup(void) {
	/*
	  Serial Port
	*/
	Serial.begin(115200);
	/*
	  LCD
	*/
	lcd.init();
	lcd.backlight();
	lcd.setCursor(0, 0);//Top line
	lcd.print("Radiator Balance");
	lcd.setCursor(0, 1);//Bottom line
	lcd.print("By Tim Jackson");
	delay(3000);
	lcd.clear();
	/*
		Sensors
			Start and Count number found.
	*/
	Sensors.begin();
	NumberOfDevices = Sensors.getDeviceCount();
	/*
		Feedback
	*/
	Serial.print("Locating devices...");
	Serial.print("Found ");
	Serial.print(NumberOfDevices, DEC);
	Serial.println(" devices.");
	for (int i = 0; i < NumberOfDevices; i++) {
		// Search the wire for address
		if (Sensors.getAddress(Addresses_of_Temperature_Sensors, i)) {
			Serial.print("Found device ");
			Serial.print(i, DEC);
			Serial.print(" with address: ");
			printAddress(Addresses_of_Temperature_Sensors);
			Serial.println();
		}
		else {
			Serial.print("Found ghost device at ");
			Serial.print(i, DEC);
			Serial.print(" but could not detect address. Check power and cabling");
		}
	}
	Sensors.setResolution(12);	//	Set Global Resalution to 12 bit.
	delay(500);
	Do_Titles();
}

void loop(void) {
	/*
		Get Temperatures
	*/
	Sensors.requestTemperatures(); // Send the command to get temperatures

	// Loop through each device, print out temperature data
	for (int i = 0; i < NumberOfDevices; i++) {
		// Search the wire for address
		if (Sensors.getAddress(Addresses_of_Temperature_Sensors, i)) {

			// Output the device ID
			Serial.print("Temperature for device: ");
			Serial.println(i, DEC);

			// Print the data
			Temperatures_inC[i] = Sensors.getTempC(Addresses_of_Temperature_Sensors);
			Serial.print("Temp C: ");
			Serial.print(Temperatures_inC[i]);
			Serial.print(" Temp F: ");
			Serial.println(DallasTemperature::toFahrenheit(Temperatures_inC[i])); // Converts tempC to Fahrenheit
		}
	}
	Display_Result();
	delay(2000);
}
/*
	Display result.
*/
void Display_Result() {

	lcd.setCursor(4, 0);
	lcd.print(Temperatures_inC[SENSOR_IN]);
	lcd.setCursor(4, 1);
	lcd.print(Temperatures_inC[SENSOR_OUT]);

	float _diff = Temperatures_inC[SENSOR_IN] - Temperatures_inC[SENSOR_OUT];

	if (_diff < TEMPERATURE_DIFFERENTIAL) {
		lcd.setCursor(11, 0);
		lcd.print("CLOSE");

	}
	else {
		lcd.setCursor(11, 0);
		lcd.print("OPEN ");
	}

	byte pos = 12;
	if (_diff < 0 ) { pos -= 1; }
	if (_diff > abs(9.99)) { pos -= 1; }
	if (_diff > abs(99.99)) { pos -= 1; }
	if (_diff > abs(999.99)) { pos -= 1; }
	lcd.setCursor(pos, 1);
	lcd.print(_diff);

}
/*
	Create LCD Titles
*/
void Do_Titles() {

	lcd.setCursor(0, 0);
	lcd.print("In =");
	lcd.setCursor(0, 1);
	lcd.print("Out=");

	//lcd.setCursor(11, 1);
	//lcd.print("Valve");
}
/*
	Print the Device Addresses in the Addresses_of_Temperature_Sensors Array
		The Array can hold up to 8 Addresses.
		There should be 2 for this application.
*/
void printAddress(DeviceAddress deviceAddress) {
	for (uint8_t i = 0; i < 8; i++) {
		if (deviceAddress[i] < 16) Serial.print("0");
		Serial.print(deviceAddress[i], HEX);
	}
}
